"""Audio module — re-exports audio types for convenience."""

from .types import (
    AlignmentResponse,
    AlignmentWord,
    AudioResponse,
    DialogueRequest,
    DialogueVoice,
    MusicClip,
    MusicRequest,
    MusicResponse,
    SoundEffectResponse,
    STTRequest,
    STTResponse,
    TTSRequest,
    TTSResponse,
    VoiceDesignResponse,
)

__all__ = [
    "AlignmentResponse",
    "AlignmentWord",
    "AudioResponse",
    "DialogueRequest",
    "DialogueVoice",
    "MusicClip",
    "MusicRequest",
    "MusicResponse",
    "SoundEffectResponse",
    "STTRequest",
    "STTResponse",
    "TTSRequest",
    "TTSResponse",
    "VoiceDesignResponse",
]
