"""Video module — re-exports video types for convenience."""

from .types import (
    GeneratedVideo,
    HeyGenAvatar,
    HeyGenTemplate,
    HeyGenVoice,
    VideoDigitalTwinRequest,
    VideoPhotoAvatarRequest,
    VideoRequest,
    VideoResponse,
    VideoStudioRequest,
    VideoTranslateRequest,
)

__all__ = [
    "GeneratedVideo",
    "HeyGenAvatar",
    "HeyGenTemplate",
    "HeyGenVoice",
    "VideoDigitalTwinRequest",
    "VideoPhotoAvatarRequest",
    "VideoRequest",
    "VideoResponse",
    "VideoStudioRequest",
    "VideoTranslateRequest",
]
